==> js-language-24.0.2-sources.jar: com/oracle/truffle/js/nodes/temporal/CreateTimeZoneMethodsRecordNode.java <==
/*
 * Copyright (c) 2022, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.temporal;

import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.js.lang.JavaScriptLanguage;
import com.oracle.truffle.js.nodes.JavaScriptBaseNode;
import com.oracle.truffle.js.nodes.access.GetMethodNode;
import com.oracle.truffle.js.runtime.JSContext;
import com.oracle.truffle.js.runtime.builtins.temporal.TimeZoneMethodsRecord;
import com.oracle.truffle.js.runtime.objects.JSDynamicObject;
import com.oracle.truffle.js.runtime.util.TemporalUtil;

/**
 * Implementation of the CreateTimeZoneMethodsRecord operation.
 */
public abstract class CreateTimeZoneMethodsRecordNode extends JavaScriptBaseNode {

    @Child private GetMethodNode getGetOffsetNanosecondsFor;
    @Child private GetMethodNode getGetPossibleInstantsFor;

    protected CreateTimeZoneMethodsRecordNode() {
        JSContext ctx = JavaScriptLanguage.get(null).getJSContext();
        this.getGetOffsetNanosecondsFor = GetMethodNode.create(ctx, TemporalUtil.GET_OFFSET_NANOSECONDS_FOR);
        this.getGetPossibleInstantsFor = GetMethodNode.create(ctx, TemporalUtil.GET_POSSIBLE_INSTANTS_FOR);
    }

    public final TimeZoneMethodsRecord executeFull(JSDynamicObject timeZone) {
        return execute(timeZone, true, true);
    }

    public final TimeZoneMethodsRecord executeOnlyGetOffsetNanosecondsFor(JSDynamicObject timeZone) {
        return execute(timeZone, true, false);
    }

    protected abstract TimeZoneMethodsRecord execute(JSDynamicObject timeZone, boolean getOffsetNanosecondsFor, boolean getPossibleInstantsFor);

    @Specialization
    protected TimeZoneMethodsRecord toRelativeTemporalObject(JSDynamicObject timeZone, boolean getOffsetNanosecondsFor, boolean getPossibleInstantsFor) {
        return new TimeZoneMethodsRecord(timeZone,
                        getOffsetNanosecondsFor ? getGetOffsetNanosecondsFor.executeWithTarget(timeZone) : null,
                        getPossibleInstantsFor ? getGetPossibleInstantsFor.executeWithTarget(timeZone) : null);
    }
}

==> js-language-24.1.1-sources.jar: com/oracle/truffle/js/nodes/temporal/CreateTimeZoneMethodsRecordNode.java <==
/*
 * Copyright (c) 2022, 2024, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.temporal;

import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.ImportStatic;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.strings.TruffleString;
import com.oracle.truffle.js.nodes.JavaScriptBaseNode;
import com.oracle.truffle.js.nodes.access.GetMethodNode;
import com.oracle.truffle.js.runtime.JSRealm;
import com.oracle.truffle.js.runtime.builtins.temporal.TimeZoneMethodsRecord;
import com.oracle.truffle.js.runtime.util.TemporalConstants;

/**
 * Implementation of the CreateTimeZoneMethodsRecord operation.
 */
@ImportStatic(TemporalConstants.class)
public abstract class CreateTimeZoneMethodsRecordNode extends JavaScriptBaseNode {

    protected CreateTimeZoneMethodsRecordNode() {
    }

    public final TimeZoneMethodsRecord executeFull(Object timeZone) {
        return execute(timeZone, true, true);
    }

    public final TimeZoneMethodsRecord executeOnlyGetOffsetNanosecondsFor(Object timeZone) {
        return execute(timeZone, true, false);
    }

    protected abstract TimeZoneMethodsRecord execute(Object timeZone, boolean getOffsetNanosecondsFor, boolean getPossibleInstantsFor);

    @Specialization
    protected TimeZoneMethodsRecord doString(TruffleString timeZone, boolean getOffsetNanosecondsFor, boolean getPossibleInstantsFor) {
        JSRealm realm = getRealm();
        return new TimeZoneMethodsRecord(timeZone,
                        getOffsetNanosecondsFor ? realm.getTemporalTimeZoneGetOffsetNanosecondsForFunctionObject() : null,
                        getPossibleInstantsFor ? realm.getTemporalTimeZoneGetPossibleInstantsForFunctionObject() : null);
    }

    @Specialization(guards = "!isString(timeZone)")
    protected TimeZoneMethodsRecord doNonString(Object timeZone, boolean getOffsetNanosecondsFor, boolean getPossibleInstantsFor,
                    @Cached("create(getJSContext(), GET_OFFSET_NANOSECONDS_FOR)") GetMethodNode getGetOffsetNanosecondsFor,
                    @Cached("create(getJSContext(), GET_POSSIBLE_INSTANTS_FOR)") GetMethodNode getGetPossibleInstantsFor) {
        return new TimeZoneMethodsRecord(timeZone,
                        getOffsetNanosecondsFor ? getGetOffsetNanosecondsFor.executeWithTarget(timeZone) : null,
                        getPossibleInstantsFor ? getGetPossibleInstantsFor.executeWithTarget(timeZone) : null);
    }
}
